@@ -1,3 +1,5 @@
+use std::convert::TryFrom;
+
 #[derive(Debug, Default)]
 pub(crate) struct Config {
     pub build: Option<String>,
@@ -261,6 +263,57 @@ pub struct Case {
     pub duration: std::time::Duration,
 }
 
+#[derive(Debug)]
+pub struct Output {
+    pub exit_code: i32,
+    pub stdout: String,
+    pub stderr: String,
+}
+
+impl Output {
+    pub fn replace(mut self, v: String) -> Self {
+        // on mac /private is added to temp folders
+        // amitu@MacBook-Pro fbt % ls /var/folders/kf/jfmbkscj7757mmr29mn3rksm0000gn/T/fbt/874862845293569866/input
+        // one
+        // amitu@MacBook-Pro fbt % ls /private/var/folders/kf/jfmbkscj7757mmr29mn3rksm0000gn/T/fbt/874862845293569866/input
+        // one
+        // both of them are the same folder, and we see the former path, but the lauched processes see the later
+
+        let private_v = format!("/private{}", v.as_str());
+        self.stdout = self.stdout.replace(private_v.as_str(), "<cwd>");
+        self.stderr = self.stderr.replace(private_v.as_str(), "<cwd>");
+        self.stdout = self.stdout.replace(v.as_str(), "<cwd>");
+        self.stderr = self.stderr.replace(v.as_str(), "<cwd>");
+
+        self
+    }
+}
+
+impl TryFrom<&std::process::Output> for Output {
+    type Error = &'static str;
+
+    fn try_from(o: &std::process::Output) -> std::result::Result<Self, Self::Error> {
+        Ok(Output {
+            exit_code: match o.status.code() {
+                Some(code) => code,
+                None => return Err("cant read exit_code"),
+            },
+            stdout: {
+                std::str::from_utf8(&o.stdout)
+                    .unwrap_or("")
+                    .trim()
+                    .to_string()
+            },
+            stderr: {
+                std::str::from_utf8(&o.stderr)
+                    .unwrap_or("")
+                    .trim()
+                    .to_string()
+            },
+        })
+    }
+}
+
 #[derive(Debug)]
 pub enum Failure {
     Skipped {
@@ -283,15 +336,19 @@ pub enum Failure {
     },
     UnexpectedStatusCode {
         expected: i32,
+        output: Output,
+    },
+    CantReadOutput {
         output: std::process::Output,
+        reason: &'static str,
     },
     StdoutMismatch {
         expected: String,
-        output: std::process::Output,
+        output: Output,
     },
     StderrMismatch {
         expected: String,
-        output: std::process::Output,
+        output: Output,
     },
     DirDiffError {
         error: crate::DirDiffError,
